package fr.natan.akkastreamfileprocessingapi.controller;

import fr.natan.akkastreamfileprocessingapi.futurecompleteness.CompletableFutureBuilder;
import fr.natan.akkastreamfileprocessingapi.models.ModelsAndJsonMap;
import fr.natan.akkastreamfileprocessingapi.service.AkkaStreamFileProcessingFuture;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import play.api.libs.json.JsValue;
import play.api.libs.json.Json;
import scala.collection.IndexedSeq;
import scala.concurrent.Future;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;


@SuppressWarnings("SpellCheckingInspection")
@RestController
public class TvSeriesController {

    private final AkkaStreamFileProcessingFuture akkaStreamFilesProcessing;
    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    public TvSeriesController(AkkaStreamFileProcessingFuture akkaStreamFilesProcessing) {
        this.akkaStreamFilesProcessing = akkaStreamFilesProcessing;
    }

    @RequestMapping(value = "/persons/id/{personID}", method = RequestMethod.GET)
    private String getPersonByID(@PathVariable(name = "personID") String personID) throws ExecutionException, InterruptedException {
        Future<ModelsAndJsonMap.Person> futurePerson = akkaStreamFilesProcessing.getPersonByIdFuture(personID);
        CompletableFuture<ModelsAndJsonMap.Person> completableFuture = CompletableFutureBuilder
                .buildcompletableFuture1(futurePerson);
        ModelsAndJsonMap.Person person = completableFuture.get();

        JsValue personJs = Json.toJson(person, person.personJsonFormatter());
        return Json.prettyPrint(personJs);
    }

    @RequestMapping(value = "/persons/name/{primaryName}", method = RequestMethod.GET)
    private String getPersonByName(@PathVariable(name = "primaryName") String primaryName)
            throws ExecutionException, InterruptedException {

        Future<ModelsAndJsonMap.Person> personFuture = akkaStreamFilesProcessing.getPersonByNameFuture(primaryName);
        CompletableFuture<ModelsAndJsonMap.Person> personCompletableFuture = CompletableFutureBuilder
                .buildcompletableFuture1(personFuture);
        ModelsAndJsonMap.Person person = personCompletableFuture.get();
        JsValue personJs = Json.toJson(person, person.personJsonFormatter());
        return Json.prettyPrint(personJs);
    }

    @RequestMapping(value = "/tvseries/id/{tvSerieId}", method = RequestMethod.GET)
    private String getTvSerieByID(@PathVariable(name = "tvSerieId") String tvSerieID)
            throws ExecutionException, InterruptedException {
        Future<ModelsAndJsonMap.TvSerie> tvSerieFuture = akkaStreamFilesProcessing.getTvSerieByIdFuture(tvSerieID);
        CompletableFuture<ModelsAndJsonMap.TvSerie> tvSerieCompletableFuture = CompletableFutureBuilder
                .buildcompletableFuture1(tvSerieFuture);

        ModelsAndJsonMap.TvSerie tvSerie = tvSerieCompletableFuture.get();
        JsValue tvSerieJs = Json.toJson(tvSerie, tvSerie.tvSerieJsonFormatter());
        return Json.prettyPrint(tvSerieJs);
    }

    @RequestMapping(value = "/tvseries/title/{tvseriePrimaryTitle}", method = RequestMethod.GET)
    private List<String> getTvserieByPrimaryTitle(
            @PathVariable(name = "tvseriePrimaryTitle") String tvseriePrimaryTitle) throws InterruptedException, ExecutionException {

        Future<IndexedSeq<ModelsAndJsonMap.TvSerie>> listFuture = akkaStreamFilesProcessing
                .getTvSeriesByPrimaryTitleFuture(tvseriePrimaryTitle);
        CompletableFuture<IndexedSeq<ModelsAndJsonMap.TvSerie>> completableFuture =
                CompletableFutureBuilder.buildcompletableFuture2(listFuture);

        IndexedSeq<ModelsAndJsonMap.TvSerie> tvSerieList = completableFuture.get();
        List<String> tvSeries = new ArrayList<>();
        tvSerieList.foreach(tvSerie -> {
            JsValue tvSerieJs = Json.toJson(tvSerie, tvSerie.tvSerieJsonFormatter());
            tvSeries.add(Json.prettyPrint(tvSerieJs));
            return null;
        });

      return tvSeries;
    }

    @RequestMapping(value = "/persons/tvseries/title/{tvSerieTitle}", method = RequestMethod.GET)
    private List<String> getPersonsForTvSerieByTvSerieTitle(@PathVariable(name = "tvSerieTitle") String tvSerieTitle)
            throws InterruptedException, ExecutionException {

        Future<IndexedSeq<ModelsAndJsonMap.Person>> futurePersonSeq = akkaStreamFilesProcessing
                .getPersonsForTvSerieByTvSerieTitleFuture(tvSerieTitle);
        CompletableFuture<IndexedSeq<ModelsAndJsonMap.Person>> completableFuture = CompletableFutureBuilder
                .buildcompletableFuture2(futurePersonSeq);

        IndexedSeq<ModelsAndJsonMap.Person> personIndexedSeq = completableFuture.get();
        List<String> personList = new ArrayList<>();
        personIndexedSeq.foreach(person -> {
            JsValue personJs = Json.toJson(person, person.personJsonFormatter());
            personList.add(Json.prettyPrint(personJs));
            return null;
        });

        return personList;
    }

    @RequestMapping(value = "/persons/tvseries/id/{tvSerieID}", method = RequestMethod.GET)
    private List<String> getPersonsForTvSerieByTvSerieID(@PathVariable(name = "tvSerieID") String tvSerieID)
            throws InterruptedException, ExecutionException {

        Future<IndexedSeq<ModelsAndJsonMap.Person>> futureIndexedSeqPersons = akkaStreamFilesProcessing
                .getPersonsForTvSerieByTvSerieIDFuture(tvSerieID);
        CompletableFuture<IndexedSeq<ModelsAndJsonMap.Person>> seqCompletableFuture = CompletableFutureBuilder
                .buildcompletableFuture2(futureIndexedSeqPersons);

        IndexedSeq<ModelsAndJsonMap.Person> personIndexedSeq = seqCompletableFuture.get();
        List<String> personsList = new ArrayList<>();
        personIndexedSeq.foreach(person ->{
            JsValue personJs = Json.toJson(person, person.personJsonFormatter());
            personsList.add(Json.prettyPrint(personJs));
            return null;
        });

        return personsList;
    }

    @RequestMapping(value = "/persons", method = RequestMethod.GET)
    private ResponseEntity<String> getAllPersons() {
        akkaStreamFilesProcessing.getAllPersonsFuture();
        return new ResponseEntity<>("is running", HttpStatus.OK);
    }

    @RequestMapping(value = "/tvseries", method = RequestMethod.GET)
    private ResponseEntity<String> getAllvSeries() {
        akkaStreamFilesProcessing.getAllTvSeriesFuture();
        return new ResponseEntity<>("is running", HttpStatus.OK);
    }
}
